/*
	yazu5407 v1.0 CPLD
*/

module cpld(
	input CLK,
//	input RT_32KHZ,
	input nRSTO,
	output nRSTI,
	output RSTO,
	input [6:0] A,
	inout [7:0] D,
	input nCS1,
	input RnW,
	input nOE,
	output nRD,
	output nWR,
	output LED0,

	output nIRQ1,
	output nIRQ3,
	output nIRQ5,
	output nIRQ7,

	input TXD0,
	output RXD0,
	input nRTS0,
	output nCTS0,
	input TXD1,
	output RXD1,
	input nRTS1,
	output nCTS1,

//	SC_nER0,
	output SC_TXD0,
	input SC_RXD0,
	output SC_nRTS0,
	input SC_nCTS0,
	input SC_VALID0,
//	output SC_FON0,
	output SC_nFOFF0,
// SC_nER1,
	output SC_TXD1,
	input SC_RXD1,
	output SC_nRTS1,
	input SC_nCTS1,
	input SC_VALID1,
//	output SC_FON1,
	output SC_nFOFF1,

	output RS_DI,
	input RS_RO,

	output IR_TX,
	input IR_RX,

	input ET_nIRQ,
	input SB_INTH,
	input SB_INTD,
	input RT_nINT,
	input nIRQA,
	input nIRQB,
	input nIRQC,
	input nIRQD,

	input nXDREQ0,
	output XDACK0,
	input nXDREQ1,
	output XDACK1,

	input SB_DREQH,
  output SB_nDACKH,
	input SB_DREQD,
  output SB_nDACKD,

	output nDREQ0,
	input TM0DACK0,
	output nDREQ1,
	input TM1DACK1
 

);

// parameter IRQ

reg [7:0] register[0:5];
wire [7:0] sm_reg, im_reg0, im_reg1, im_reg2, im_reg3, dm_reg;
wire [1:0] sm0_sel, sm1_sel;
reg sm0_nen, sm1_nen;
wire [2:0] im0_sel, im1_sel, im2_sel, im3_sel; 
wire im0_nen, im1_nen, im2_nen, im3_nen;
wire [2:0] dm0_sel, dm1_sel;
reg dm0_nen, dm1_nen;

assign RSTO = !nRSTO;
assign nRSTI = 'bz;
assign LED0 = nRSTO;

assign nRD = !RnW;
assign nWR = RnW;

assign D = (!nOE && RnW && !nCS1) ? register[A] : 'bz;
always @(posedge CLK)
begin
	if (!nCS1 && !RnW) register[A] = D; // load data register
end

// IRQ mux
assign im_reg0 = register[1];
assign im0_sel = im_reg0[2:0];
assign im0_nen = im_reg0[3];
assign im_reg1 = register[2];
assign im1_sel = im_reg1[2:0];
assign im1_nen = im_reg1[3];
assign im_reg2 = register[3];
assign im2_sel = im_reg2[2:0];
assign im2_nen = im_reg2[3];
assign im_reg3 = register[4];
assign im3_sel = im_reg3[3:0];
assign im3_nen = im_reg3[3];

irqmux im0(
	.sel(im0_sel),
	.nen(im0_nen),
	.nirq0(ET_nIRQ),
	.nirq1(SB_INTH),	 
	.nirq2(SB_INTD),	 
	.nirq3(RT_nINT),	 
	.nirq4(nIRQA),	 
	.nirq5(nIRQB),	 
	.nirq6(nIRQC),	 
	.nirq7(nIRQD),	 
  .nirq(nIRQ1)
);

irqmux im1(
	.sel(im1_sel),
	.nen(im1_nen),
	.nirq0(ET_nIRQ),
	.nirq1(SB_INTH),	 
	.nirq2(SB_INTD),	 
	.nirq3(RT_nINT),	 
	.nirq4(nIRQA),	 
	.nirq5(nIRQB),	 
	.nirq6(nIRQC),	 
	.nirq7(nIRQD),	 
  .nirq(nIRQ3)
);

irqmux im2(
	.sel(im2_sel),
	.nen(im2_nen),
	.nirq0(ET_nIRQ),
	.nirq1(SB_INTH),	 
	.nirq2(SB_INTD),	 
	.nirq3(RT_nINT),	 
	.nirq4(nIRQA),	 
	.nirq5(nIRQB),	 
	.nirq6(nIRQC),	 
	.nirq7(nIRQD),	 
  .nirq(nIRQ5)
);

irqmux im3(
	.sel(im3_sel),
	.nen(im3_nen),
	.nirq0(ET_nIRQ),
	.nirq1(SB_INTH),	 
	.nirq2(SB_INTD),	 
	.nirq3(RT_nINT),	 
	.nirq4(nIRQA),	 
	.nirq5(nIRQB),	 
	.nirq6(nIRQC),	 
	.nirq7(nIRQD),	 
  .nirq(nIRQ7)
);


// DMA mux
assign dm_reg = register[5];
assign dm0_sel = dm_reg[2:0];
// assign dm0_nen = dm_reg[3];
assign dm1_sel = dm_reg[6:4];
// assign dm1_nen = dm_reg[7];
wire SB_DACKH, SB_DACKD;
assign SB_nDREQH = !SB_DREQH;
assign SB_nDACKH = !SB_DACKH;
assign SB_nDREQD = !SB_DREQD;
assign SB_nDACKD = !SB_DACKD;

always @*
begin
if (dm0_sel == dm1_sel) // collision
	begin
	dm0_nen <= 1'b1;
  dm1_nen <= 1'b1;
	end
else
	begin
	dm0_nen <= dm_reg[3];
  dm1_nen <= dm_reg[7];
	end
end

dmamux dm0(
	.sel(dm0_sel),
	.nen(dm0_nen),
	.ndreq0(SB_nDREQH),
	.dack0(SB_DACKH),
	.ndreq1(SB_nDREQD),
	.dack1(SB_DACKD),
	.ndreq2(nXDREQ0),
	.dack2(XDACK0),
	.ndreq3(nXDREQ1),
	.dack3(XDACK1),
	.ndreq4(1'b1), // FPGA
//	.dack4(),
	.ndreq(nDREQ0),
	.dack(TM0DACK0)
);

dmamux dm1(
	.sel(dm1_sel),
	.nen(dm1_nen),
	.ndreq0(SB_nDREQH),
	.dack0(SB_DACKH),
	.ndreq1(SB_nDREQD),
	.dack1(SB_DACKD),
	.ndreq2(nXDREQ0),
	.dack2(XDACK0),
	.ndreq3(nXDREQ1),
	.dack3(XDACK1),
	.ndreq4(1'b1), // FPGA
//	.dack4(),
	.ndreq(nDREQ1),
	.dack(TM1DACK1)
);



// SCI mux
assign sm_reg = register[0];
assign sm0_sel = sm_reg[1:0];
// assign sm0_nen = sm_reg[2];
assign sm1_sel = sm_reg[4:3];
// assign sm1_nen = sm_reg[5];

always @*
begin
if (sm0_sel == sm1_sel) // collision
	begin
	sm0_nen <= 1'b1;
  sm1_nen <= 1'b1;
	end
else
	begin
	sm0_nen <= sm_reg[2];
  sm1_nen <= sm_reg[5];
	end
end

scimux sm0(
	.sel(sm0_sel),
	.nen(sm0_nen),
	.sca_txd(SC_TXD0),
	.sca_rxd(SC_RXD0),
	.sca_nrts(SC_nRTS0),
	.sca_ncts(SC_nCTS0),
	.scb_txd(SC_TXD1),
	.scb_rxd(SC_RXD1),
	.scb_nrts(SC_nRTS1),
	.scb_ncts(SC_nCTS1),
	.rs_txd(RS_DI),
	.rs_rxd(RS_RO),
	.ir_txd(IR_TX),
	.ir_rxd(IR_RX),
	.txd(TXD0),
	.rxd(RXD0),
	.nrts(nRTS0),
	.ncts(nCTS0)
);

scimux sm1(
	.sel(sm1_sel),
	.nen(sm1_nen),
	.sca_txd(SC_TXD0),
	.sca_rxd(SC_RXD0),
	.sca_nrts(SC_nRTS0),
	.sca_ncts(SC_nCTS0),
	.scb_txd(SC_TXD1),
	.scb_rxd(SC_RXD1),
	.scb_nrts(SC_nRTS1),
	.scb_ncts(SC_nCTS1),
	.rs_txd(RS_DI),
	.rs_rxd(RS_RO),
	.ir_txd(IR_TX),
	.ir_rxd(IR_RX),
	.txd(TXD1),
	.rxd(RXD1),
	.nrts(nRTS1),
	.ncts(nCTS1)
);

// SC power, select autoshutdown or poweroff
SC_nFOFF0 = (sm0_sel == 2'b00 && !sm0_nen || sm1_sel == 2'b00 && !sm1_nen)
SC_nFOFF1 = (sm0_sel == 2'b01 && !sm0_nen || sm1_sel == 2'b01 && !sm1_nen)


endmodule
