/*
	yazu serial mux
	Muxes SC0, SC1, RS485 and IrDA to a yazu UART
*/

module scimux(
// control
	input [1:0] sel,
	input nen,

// source
	output reg sca_txd,
	input sca_rxd,
	output reg sca_nrts,
	input sca_ncts,

	output reg scb_txd,
	input scb_rxd,
	output reg scb_nrts,
	input scb_ncts,

	output reg rs_txd,
	input rs_rxd,

	output reg ir_txd,
	input ir_rxd,

// target
	input txd,
	output reg rxd,
	input nrts,
	output reg ncts
);

always @*
begin
if (nen) // disabled
begin
	sca_txd <= 'bz;
	sca_nrts <= 'bz;
	scb_txd <= 'bz;
	scb_nrts <= 'bz;
	rs_txd <= 'bz;
	ir_txd <= 'bz;
	rxd <= 'bz;
	ncts <= 'bz;
end
else // enabled
begin
	case (sel)
	2'b00: // uart <-> sca
	begin
	sca_txd <= txd;
	rxd <= sca_rxd;
	sca_nrts <= nrts;
	ncts <= sca_ncts;

	scb_txd <= 'bz;
	scb_nrts <= 'bz;
	rs_txd <= 'bz;
	ir_txd <= 'bz;
	end

	2'b01: // uart <-> scb
	begin
	scb_txd <= txd;
	rxd <= scb_rxd;
	scb_nrts <= nrts;
	ncts <= scb_ncts;

	sca_txd <= 'bz;
	sca_nrts <= 'bz;
	rs_txd <= 'bz;
	ir_txd <= 'bz;
	end

	2'b10: // uart <-> rs
	begin
	rs_txd <= txd;
	rxd <= rs_rxd;

	sca_txd <= 'bz;
	sca_nrts <= 'bz;
	scb_txd <= 'bz;
	scb_nrts <= 'bz;
	ir_txd <= 'bz;
	ncts <= 'bz;
	end

	2'b11: // uart <-> ir
	begin
	ir_txd <= txd;
	rxd <= ir_rxd;

	sca_txd <= 'bz;
	sca_nrts <= 'bz;
	scb_txd <= 'bz;
	scb_nrts <= 'bz;
	rs_txd <= 'bz;
	ncts <= 'bz;
	end
	endcase
end
end

endmodule
